function [frqs] = ldfm_frqinit_dis(X, F0, ndims, fw, opt)

%[frqs] = ldfm_frqinit_dis(X, F0, ndims, fw, opt)
%
% This function implements a binary search for a miximg parameter gamma,
% so that the output dimensionality of the feature map matches ndims
%
% For more details see Chum: Low Dimensional Explicit Feature Maps, ICCV'15
% http://cmp.felk.cvut.cz/~chum
% (C) Ondrej Chum 2015

wrange = [0, inf];
if nargin < 4
  fw = 10;
end
dimdiff = inf;
fiterror = inf;

opt.do_sparse = 1;

optF = opt;
optF.do_sparse = 0;
optF.do_taylor = 0;

for i = 1 : 20
  opt.fit_weight = fw;  
  [w, err, ~, F] = ldfm_fitfun(X, F0, opt);
  
  if isempty(w) % linear program failed for some reason
     fw = (fw + wrange(1)) / 2;
     continue;
   end
  
  keep = find(w > max(w)/1000);
  
  
 if size(F,1) == 1 
     nF = [];
     nw = [];
     i = 1;
     while i < length(keep)

       if keep(i)+1 == keep(i+1)
         a = w(keep(i:i+1));
          nw = [nw; sum(a)];
         a = a / sum(a);
         nF = [nF,a(1)*F(keep(i))+a(2)*F(keep(i)+1)];
          i = i + 2;
       else
         nF = [nF, F(keep(i))];
         nw =[nw; w(keep(i))];
         i = i + 1;
       end
     end
 else
     nF = F(:,keep);
     nw = w(keep);
 end

  dm = ldfm_frq2dim(nF);
  odim = sum(dm);
  fprintf('Sample dim:  %d  fit weight %.4f\n',odim, fw);
 
  
  udim = odim;
  
     if odim > ndims
           [~,ord] = sort(nw, 'descend');
           cdm = cumsum(dm(ord));
           pos = find(cdm <= ndims);
           if ~isempty(pos)
               pos = pos(end);
               nF = nF(:,ord(1:pos));
               [~,reord] = sort(sum(nF,1));
               nF = nF(:,reord);
               udim = cdm(pos);
           end
     end
     
     if (udim < ndims) & (all(sum(nF,1) > 0))
         nF = [zeros(size(nF,1),1),nF];
         udim = udim + 1;
     end
 
         [w, err] = ldfm_fitfun(X, nF, optF);
        if (err < fiterror)
            fiterror = err;
            dimdiff = 0;
            fit_weight = fw;
            frqs = nF;
        end
        
  
  if odim <= ndims
      wrange(1) = fw;
      if isinf(wrange(2))
          fw = 2*fw;
      else
          fw = (fw + wrange(2)) / 2;
      end
  else
      wrange(2) = fw;
      fw = (fw + wrange(1)) / 2;
  end

end