function [F, w, X] = ldfm_kernelsignature(X, k, F0, outdim, opt)

%[F, w, X] = ldfm_kernelsignature(X, k, F0, outdim, opt)
%
% approximates kernel signature function k by up to outdim dimensional
% feature map on diecrete input values specified in X.d{i}.x
% using a pool of frequencies F0
%
% function y = k(x) takes d x n array, where d is an input dimension and
%  n is a number of points and returns 1 x n kernel signature response
%
% Output: F frequencies, w weights, X datastructure with some
% internal data filled in
%
% For more details see Chum: Low Dimensional Explicit Feature Maps, ICCV'15
% http://cmp.felk.cvut.cz/~chum
% (C) Ondrej Chum 2015

aopt.max_frq_iter    = 20;
aopt.frq_delta_multi = 1.1;
aopt.gamma_weight = 2;

if nargin < 5
    opt = aopt;
else
    opt = optmerge(opt, aopt);
end

optT.delta_div = 2;
optT.do_sparse = 1;
optT.do_taylor = 0;

X = ldfm_dataprepare(X, k); %prepare domain and values
F = ldfm_frqinit_dis(X, F0, outdim, opt.gamma_weight, optT);

% Iterate continuous frequency estimate

optI = optT;
optI.do_sparse = 0;
optI.do_taylor = 1;

for i = 1:opt.max_frq_iter
    optI.delta_div = optI.delta_div * opt.frq_delta_multi;
    [wi, err, Ci, F] = ldfm_fitfun(X, F, optI);
end
    
% Final Fit

optF = optT;
optF.do_sparse = 0;
optT.do_taylor = 0;

[w, err, C] = ldfm_fitfun(X,F,optF);